<?php
class Custom_ComputerSelect
{
    function getRulesTree ($task_id)
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->from("rules")->where('task_id = ?', $task_id);
        $fetch = $db->fetchAssoc($select);
        $result = array();
        foreach ($fetch as $row) {
            $row["text_string"] = "";
            if ($row["user_match"] != null) {
                //getUsers
                $mainMatch = "USERS";
                $V = $this->findMatches($task_id, $row["rule_id"], $mainMatch);
                $row["text_string"] = count($V) . " Users on ";
                $match = $row["user_match"];
            }
            if ($row["computer_match"] != null) {
                // Get comps
                if ($row["text_string"] == "")
                    $mainMatch = "COMPUTERS";
                $match = $row["computer_match"];
                $row["text_string"] .= "  Computer(s) $row[computer_match] in";
            }
                        
            if ($row["workgroup_match"] != null && $row["domain_match"] == null) {
                if ($row["text_string"] == "")
                    $mainMatch = "WORKGROUP";
                $match = $row["workgroup_match"];
                $row["text_string"] .= " Workgroup(s) $row[workgroup_match] ";
            }
            if ($row["domain_match"] != null && $row["workgroup_match"] == null) {
                if ($row["text_string"] == "")
                    $mainMatch = "DOMAINS";
                $match = $row["domain_match"];
                $row["text_string"] .= " Domain(s) $row[domain_match]";
            }
            if ($row["domain_match"] != null && $row["workgroup_match"] != null) {
                if ($row["text_string"] == "")
                    $mainMatch = "WORKGROUP";
                $match = $row["domain_match"];
                $row["text_string"] .= " Domain(s) $row[domain_match]";
            }
            if ($row["domain_match"] == null && $row["workgroup_match"] == null) {
                if ($row["text_string"] == "")
                    $mainMatch = "WORKGROUP";
                $row["text_string"] .= " All Workgroups or Domains";
            }
            
            if ($row["ip_match"] != NULL) {
                $row["text_string"] .= " With IP $row[ip_match]";
            }
            
            $row["matchType"] = $mainMatch;
            $row["children"] = $this->findMatches($task_id, $row["rule_id"], $mainMatch);
            $result[] = $row;
        }
        return $result;
    }
    /**
     * Finds child matches for task rules.
     *
     * @param int $task_id
     * @param int $rule_id
     * @param stringCase $matchType
     * @return array
     */
    function findMatches ($task_id, $rule_id, $matchType)
    {
        $db = Zend::registry('dbController');
        /* Get the main Search values */
        /* Columns combined are ANDS, commas are ORs, ^ are nots */
        $select = $db->select()->from('rules')->where('rule_id = ?', $rule_id)->where("task_id = ?", $task_id);
        $rows = $db->fetchAssoc($select);
        $childSelect = $db->select();
        /* Parse trough values */
        foreach ($rows as $row) {
            if ($matchType != "USERS") {
                $childSelect->from('computers')->where('status = ?', 'approved');
                foreach ($row as $key => $value) {
                    $row["$key"] = trim($value);
                }
                if ($row["computer_match"] != null)
                    $this->makeSelect($childSelect, $row["computer_match"], "computer_name");
                if ($row["workgroup_match"] != null)
                    $this->makeSelect($childSelect, $row["workgroup_match"], "workgroup");
                if ($row["domain_match"] != null)
                    $this->makeSelect($childSelect, $row["domain_match"], "domain");
                if ($row["ip_match"] != null)
                    $this->makeSelect($childSelect, $row["ip_match"], "ip");
            } else {
                $childSelect->from("computers")->join("users", "users.computer_id = computers.comp_guid");
                if ($row["computer_match"] != null)
                    $this->makeSelect($childSelect, $row["computer_match"], "computers.computer_name");
                if ($row["workgroup_match"] != null)
                    $this->makeSelect($childSelect, $row["workgroup_match"], "computers.workgroup");
                if ($row["domain_match"] != null)
                    $this->makeSelect($childSelect, $row["domain_match"], "computers.domain");
                if ($row["ip_match"] != null)
                    $this->makeSelect($childSelect, $row["ip_match"], "computers.ip");
                if ($row["user_match"] != null)
                    $this->makeSelect($childSelect, $row["user_match"], "users.name");
            }
        }
        try {
            $result = $db->fetchAssoc($childSelect);
        } catch (Exception $e) {
            Zend::dump($e);
        }
        $children = array();
        foreach ($result as $row) {
            if ($matchType == "USERS") {
                $children[] = array("name" => $row["name"] , "id" => $row["id"] , "rule_id" => $rule_id);
            } else {
                $children[] = array("name" => $row["computer_name"] , "id" => $row["comp_guid"] , "rule_id" => $rule_id);
            }
        }
        return $children;
    }
    function makeSelect (&$select, $data, $field, $firstAnd = true)
    {
        $vData = explode(',', trim($data));
        $fData = array();
        $max = count($vData);
        $i = 1;
        $positives = array();
        $negatives = array();
        foreach ($vData as $element) {
            if ($element == "")
                continue;
            if (substr($element, 1, 1) == "^") {
                $match = "!=";
                $element = trim(substr($element, 2));
            } else {
                $match = "=";
            }
            if (strstr($element, "*")) {
                $element = trim(strtr($element, "*", "%"));
                $match2 = "LIKE";
            } else {
                $match2 = "=";
            }
            $element = trim($element);
            //if ($match == "");
            // $match1 == NOT
            // $match2 = / LIKE
            // NOT LIKE, !=, =, LIKE
            if ($match == "=" && $match2 == "LIKE")
                $fMatch = "LIKE";
            if ($match == "!=" && $match2 == "LIKE")
                $fMatch = "NOT LIKE";
            if ($match == "=" && $match2 == "=")
                $fMatch = "=";
            if ($match == "!=" && $match2 == "=")
                $fMatch = "!=";
            if ($fMatch == "!=" || $fMatch == "NOT LIKE") {
                $negatives[] = "$field $fMatch " . Zend::registry("dbController")->quote("$element");
            } else {
                $positives[] = "$field $fMatch " . Zend::registry("dbController")->quote("$element");
            }
        }
        $nWheres = '(' . implode(" AND ", $negatives) . ')';
        $pWheres = '(' . implode(" OR ", $positives) . ')';
        $fWhere = "(" . $pWheres . ($nWheres != "()" ? " AND $nWheres" : "") . ")";
        if ($fWhere == "()")
            die("SOMETHING BAD HAPPENED! RLY BAD!");
        $select->where("$fWhere");
    }
    function getWorkgroupTree ()
    {
        $db = Zend::registry('dbController');
        /* Step 1: Get all workgroups */
        $workgroups = $this->getWorkgroups();
        $fWorkgroups = array();
        foreach ($workgroups as $workgroup) {
            $computers = $this->getComputersInWorkgroup($workgroup);
            $fComputers = array();
            foreach ($computers as $computer) {
                $users = $this->getUsersInComputer($computer, $workgroup);
                $fComputers = array_merge($fComputers, array("$computer" => $users));
            }
            $fWorkgroups = array_merge($fWorkgroups, array("$workgroup" => $fComputers));
        }
        return $fWorkgroups;
    }
    function getWorkgroupTreeWithKeys ()
    {
        $db = Zend::registry('dbController');
        /* Step 1: Get all workgroups */
        $workgroups = $this->getWorkgroups();
        $fWorkgroups = array();
        foreach ($workgroups as $workgroup) {
            $computers = $this->getComputersAndKeysInWorkgroup($workgroup);
            $fComputers = array();
            foreach ($computers as $computer) {
                $fComputers[] = array("compname" => "$computer[computer_name]" , "compguid" => $computer["comp_guid"]);
            }
            $fWorkgroups = array_merge($fWorkgroups, array("$workgroup" => $fComputers));
        }
        return $fWorkgroups;
    }
    function getDomainTree ()
    {
        $db = Zend::registry('dbController');
        /* Step 1: Get all workgroups */
        $workgroups = $this->getDomains();
        $fWorkgroups = array();
        foreach ($workgroups as $workgroup) {
            $computers = $this->getComputersInDomains($workgroup);
            $fComputers = array();
            foreach ($computers as $computer) {
                $users = $this->getUsersInComputer($computer, $workgroup);
                $fComputers[] = array("$computer" => $users);
            }
            $fWorkgroups[] = array("$workgroup" => $fComputers);
        }
        return $fWorkgroups;
    }
    function getWorkgroups ()
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('computers', "workgroup")->distinct();
        $result = $db->fetchCol($select);
        return $result;
    }
    function getDomains ()
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('computers', 'domain')->distinct();
        $result = $db->fetchCol($select);
        return $result;
    }
    function getComputersInDomains ($domain)
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->distinct()->from('computers', 'computer_name')->where('domain = ?', $domain);
        $result = $db->fetchCol($select);
        return $result;
    }
    function getComputersInWorkgroup ($workgroup)
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->distinct()->from('computers', "computer_name")->where('workgroup = ?', $workgroup)->where('status = ?', 'approved');
        $result = $db->fetchCol($select);
        return $result;
    }
    function getComputersAndKeysInWorkgroup ($workgroup)
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->distinct()->from('computers', array("computer_name" , "comp_guid"))->where('workgroup = ?', $workgroup)->where('status = ?', 'approved');
        $result = $db->fetchAll($select);
        return $result;
    }
    function getUsersInComputer ($compName, $workgroup)
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->distinct()->from('users', 'name')->join("computers", "users.computer_id = computers.comp_guid")->where('computer_name = ?', $compName)->where('workgroup = ?', $workgroup);
        $result = $db->fetchCol($select);
        return $result;
    }
    function getUserTasks ($computerName, $workgroup)
    {
        /* Select Non Admin Tasks */
        $db = Zend::registry('dbController');
        $compUid = $db->fetchCol("SELECT comp_guid FROM computers WHERE computer_name = :name AND workgroup = :workgroup", array("name" => $computerName , "workgroup" => $workgroup));
        $compUid = $compUid[0];
        $tasks = $db->select()->from('tasks')->where('owner = ?', $compUid);
        $oTasks = $db->fetchAll($tasks);
        return $oTasks;
    }
    function getAdminTasks ($computerName, $workgroup, $domain = NULL)
    {
        $db = Zend::registry('dbController');
        /*$select = $db->select()->from('rules',"task_id")->where("(computer_match LIKE \"%$computerName%\" AND computer_match NOT LIKE \"%^$computerName%\")");
        $select->where("(workgroup_match LIKE \"%$workgroup%\" AND workgroup_match NOT LIKE \"%^$workgroup%\")")->distinct();
        $result = $db->fetchAll($select);

        if (count($result) <= 0) {
        $select = $db->select()->from('rules', 'task_id')->where("(computer_match LIKE \"%$computerName%\" AND computer_match NOT LIKE \"%^$computerName%\")");
        $select->orWhere("(workgroup_match LIKE \"%$workgroup%\" AND workgroup_match NOT LIKE \"%^$workgroup%\")")->distinct();
        $result = $db->fetchAll($select);
        }*/
        // Get a list of all the rules
        /* Bug: #2111,#2110,#2109 (Se selecteaza calculatoarele cu nume asemanator */
        $resultTemporary = array();
        $result1 = array();
        $select = $db->select()->from('rules')->where('(computer_match LIKE "%' . $computerName . '%" AND computer_match NOT LIKE "^' . $computerName . '")');
        $select->where("(workgroup_match LIKE \"%$workgroup%\" AND workgroup_match NOT LIKE \"%^$workgroup%\")");
        $select->distinct();
        $resultTemporary = $db->fetchAll($select);
        /*FIX: Fix for bug 2111,2110,2109*/
        foreach ($resultTemporary as $rowTemp) {
            $computerList = explode(',', $rowTemp['computer_match']);
            $workgroupList = explode(',', $rowTemp['workgroup_match']);
            if (((array_search($computerName, $computerList) !== false) && (array_search($workgroup, $workgroupList) !== false))) {
                $result1[] = $rowTemp;
            }
        }
        $select = $db->select()->from('rules')->where('(computer_match IS NULL)');
        $select->where("(workgroup_match LIKE \"%$workgroup%\" AND workgroup_match NOT LIKE \"%^$workgroup%\")");
        $select->distinct();
        $result2 = $db->fetchAll($select);
        $result = array_merge_recursive($result1, $result2);
        if (! $result)
            return false;
        $oTasks = array();
        foreach ($result as $row) {
            $x = $db->fetchRow('SELECT * FROM tasks WHERE id = :id AND admin_task = "Yes" AND task_hidden="No"', array("id" => $row["task_id"]));
            if ($x == false)
                continue;
            $oTasks[] = $x;
        }
        return $oTasks;
    }
    function smartyGetUserTasks ($params, &$smarty)
    {
        $computerName = $params['computer_name'];
        $userTasks = $this->getUserTasks($computerName, $params["workgroup"]);
        $adminTasks = $this->getAdminTasks($params['computer_name'], $params["workgroup"]);
        if (! $adminTasks)
            $tasks = $userTasks; else
            $tasks = array_merge($userTasks, $adminTasks);
        $smarty->assign("computerTasks", $tasks);
    }
    function smartyGetComputerName ($params, &$smarty)
    {
        $id = $params['id'];
        $db = Zend::registry('dbController');
        $select = $db->select()->from('computers')->where('comp_guid = ?', $id)->limit(1);
        $result = $db->fetchRow($select);
        return $result['computer_name'];
    }
    function smartyGetComputerStatus ($params, &$smarty)
    {
        $name = $params['name'];
        $workgroup = $params['workgroup'];
        $db = Zend::registry('dbController');
        $select = $db->select()->from('computers')->where('computer_name = ?', $name)->where('workgroup = ?', $workgroup);
        $result = $db->fetchRow($select);
        $refresh_time = Options::get('refresh_time') + 60;
        $smarty->assign('computerStatus', $result["last_update"] + $refresh_time < time(0));
    }
    function getTaskComputers ($task_id)
    {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks')->where('id = ?', $task_id);
        $row = $db->fetchRow($select);
        if ($row["admin_task"] == "No") {
            $children[] = $row["owner"];
            return $children;
        }
        $children = array();
        $taskTree = $this->getRulesTree($task_id);
        foreach ($taskTree as $rule) {
            if ($rule['matchType'] != 'USERS') {
                $nChildren = $rule["children"];
                foreach ($nChildren as $child) {
                    $children[] = $child['id'];
                }
            }
        }
        return $children;
    }
    function smartyGetTasksChildren ($params, &$smarty)
    {
        $task_id = $params['task_id'];
        $smarty->assign('taskChildren', $this->getTaskComputers($task_id));
        return;
    }
    function smartyGetTasksRunning ($params, &$smarty)
    {
        $task_gid = $params["task_gid"];
        $db = Zend::registry('dbController');
        $select = $db->select()->from('task_status')->where('task_status != ?', 'Stopped');
        $select->where('task_guid = ?', $task_gid);
        $fetch = $db->fetchAll($select);
        $smarty->assign('taskStatus', $fetch);
        return;
    }
    function smartyArraySearch ($params, &$smarty)
    {
        $xsearch = array_search($params['search'], $params['array']);
        if ($xsearch !== false)
            return "checked";
        return;
    }
    function smartySeparateString ($string)
    {
        return implode('; ', explode(";", $string));
    }
}
