<?php
/**
 * The cSelect task management parser.
 * 
 * @author  Nophinity
 * @version  1.0
 * @todo Make some real error reporting system.
 *
 */
class Custom_ParseCommands
{
    /**
     * The raw text, unparsed, just how its received
     *
     * @var string
     */
    private $rawText;
    /**
     * An array that's containing all the commands that will be executed to the sql server.
     *
     * @var array
     */
    private $commands;
    /**
     * Error code, for debuging and error printing porposes.
     *
     * @var int
     */
    public $errorCode;
    /**
     * The errorline containing description of the error.
     *
     * @var string
     */
    public $errorLine;
    /**
     * The holder for the SQL select controller
     *
     * @var Zend_DB
     */
    private $dbC;
    /**
     * Marks that an error has occured.
     *
     * @param int $code
     * @param string $line
     */
    private function setError ($code, $line)
    {
        $this->errorCode = $code;
        $this->errorLine = $line . "\n";
        return;
    }
    /**
     * Checks for errors, if it finds them, it dies:p
     *
     */
    private function checkError ()
    {
        if ($this->errorCode > 0)
            return true;
    }
    /**
     * The class construct
     *
     * @param string $input
     * @param Zend_DB $db
     */
    function __construct ($input, $db = 0)
    {
        $this->dbC = $db;
        $this->rawText = trim($input);
    }
    /**
     * The main parse function, delimits, runs and parses code.
     *
     * @return array
     */
    public function parse ()
    {
        $input = $this->rawText;
        if ($input == "") {
            return false;
        }
        /* Find the first delimiter (if none print error) */
        $oldPos = 0;
        $curPos = 0;
        $intLine = 0;
        while ($curPos !== false) {
            $curPos = strpos($input, ";", $oldPos);
            $string = substr($input, $oldPos, ($curPos - $oldPos));
            if ($string == "")
                continue;
            $oldPos = $curPos + 1;
            $this->commands[] = strtolower(trim($string));
            $intLine ++;
        }
        $result = $this->parseCommands($this->commands);
        if ($this->checkError() == true)
            return false;
        return $result;
    }
    /**
     * Parses individual commands and sends them to execCommand
     *
     * @param array $commands
     * @return array
     */
    private function parseCommands ($commands)
    {
        $multi = array();
        foreach ($commands as $line => $command) {
            $primaryCommand = explode(' ', $command, 2);
            $execCommand = $primaryCommand[0];
            $execArgv = explode(' ', $primaryCommand[1]);
            $execArgc = count($execArgv);
            $multi[] = $this->createQuery($execCommand, $execArgv, $execArgc, $primaryCommand[1]);
        }
        return $multi;
    }
    /**
     * Parses the arguments of a function and turns them into proper arrays.
     *
     * @param string $args
     * @return array
     */
    private function parseArgs ($args)
    {
        $startString = strpos($args, "(");
        $endString = strrpos($args, ")");
        if ($startString == false)
            return - 1;
        $string = substr($args, $startString + 1, ($endString - $startString - 1));
        $argvs = explode(' ', $string);
        $remadeArgs = array();
        foreach ($argvs as $id => $value) {
            $value = explode(",", $value);
            if (count($value) > 1)
                $remadeArgs[] = $value; else
                $remadeArgs[] = $value[0];
        }
        return $remadeArgs;
    }
    /**
     * Creates a SQL query based on the command arguments.
     *
     * @param string $command
     * @param array $argv
     * @param int $argc
     * @param strings $args
     * @return Zend_Db_Select
     */
    private function createQuery ($command, $argv, $argc, $args)
    {
        $db = $this->dbC;
        $select = $db->select();
        switch ($command) {
            case "all":
                if ($argv[0] === "computers" && $argv[1] == "in" && $argv[2] == "network")
                    $select->from('computers'); elseif ($argv[0] === "computers" && $argv[1] == "in" && $argv[2] == "workgroup") {
                    $select->from('computers');
                    $select->where('workgroup = ?', $argv[3]);
                    $argz = $this->parseArgs($args);
                    $this->loopCommand($argz, $select, "workgroup");
                } elseif ($argv[0] === "computers" && $argv[1] == "in" && $argv[2] == "domain") {
                    $select->from('computers');
                    $argz = $this->parseArgs($args);
                    $this->loopCommand($argz, $select, "domain");
                } else
                    $this->setError(43, "Unknown arguments cmd '$command'.");
                break;
            case "computers":
                if ($argv[0] == "named") {
                    $select->from('computers');
                    $argz = $this->parseArgs($args);
                    $this->loopCommand($argz, $select, "name");
                } elseif ($argv[0] == "with" && $argv[1] == "ip") {
                    $select->from('computers');
                    $argz = $this->parseArgs($args);
                    $this->loopCommand($argz, $select, "ip");
                } else {
                    $this->setError(46, "Unknown arguments cmd '$command'.");
                }
                break;
            default:
                $this->setError(47, "Unknown command '$command'.");
                break;
        }
        $this->checkError();
        return trim($select->__toString());
    }
    /**
     * Loops trough the argument field and inserts the where and other selects
     * basicly it stacks a list and modified the select object.
     *
     * @param array $argz
     * @param Zend_DB_Select $select
     * @param string $field
     */
    function loopCommand ($argz, &$select, $field)
    {
        foreach ($argz as $id => $arg) {
            if (is_array($arg)) {
                foreach ($arg as $kId => $subArg) {
                    $match = "=";
                    if (substr_count($subArg, '*') > 0)
                        $match = "LIKE";
                    if (substr_count($subArg, '^') > 0)
                        $match = "NOT LIKE";
                    $subArg = str_replace("^", "", $subArg);
                    $subArg = str_replace("*", "%", $subArg);
                    $select->orWhere("$field $match ?", $subArg);
                }
            } else {
                $match = "=";
                if (substr_count($arg, '*') > 0)
                    $match = "LIKE";
                if (substr_count($arg, '^') > 0)
                    $match = "NOT LIKE";
                $arg = str_replace("^", "", $arg);
                $arg = str_replace("*", "%", $arg);
                $select->orWhere("$field $match ?", $arg);
            }
        }
    }
}