<?php
class Custom_TaskManagement {

    /**
     * Serializes and creates a object
     *
     * @param makeObject array $object
     * @return string
     */
    public static function makeObject($object) {
        $serializedObject = serialize($object);
        return base64_encode($serializedObject);
    }

    /**
     * Unserializes and loads a object
     *
     * @param string $object
     * @return array
     */
    public static function loadObject($object) {
        $serializedObject = base64_decode($object);
        $unObject = unserialize($serializedObject);
        return $unObject;
    }

    /**
     * Generets a UUID by the ISO standard RTF
     *
     * @return string
     */
    public static function uuid() {
        return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
        mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),
        mt_rand( 0, 0x0fff ) | 0x4000,
        mt_rand( 0, 0x3fff ) | 0x8000,
        mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ) );
    }

    /**
     * Adds a serialized task object to the database
     *
     * @param makeObject $object
     * @return int $numberOfRows
     */
    public static function addTask($object,$special=false) {
        $db = Zend::registry('dbController');

        $unseralizedObject = self::loadObject($object);
        $taskName = $unseralizedObject["task_general"]["task_name"];
        $userId = 0;
        $adminTask = $_SESSION["isAdmin"]?"Yes":"No";

        $uuid = self::uuid();
        $unseralizedObject["uuid"] = $uuid;
        $object = self::makeObject($unseralizedObject);

        $data_md5 = md5($object);
        if ($special != false) {
            $select = $db->select()->from('tasks')
            ->where('data_md5 = ?', $data_md5)
            ->orWhere('task_name = ?', $taskName);

            $result = $db->fetchAssoc($select);

            if (count($result) > 0) { return -1; }
        }
        $row = array(
        "guid" => $uuid,
        "user_id" => $userId,
        "data" => $object,
        "data_md5" => md5($object),
        "task_name" => $taskName,
        "admin_task" => $adminTask,
        "task_type" => $unseralizedObject["task_destination"]["backupDestinationType"],
        "create_date" => time(0)
        );
        $db->insert('tasks', $row);
        return $db->lastInsertId();
    }

    /**
     * Modifies a task and updates it in the database
     *
     * @param int $id
     * @param makeObject $object
     * @return int $numberOfAffectedRows
     */
    public static function updateTask($id, $object) {
        $db = Zend::registry('dbController');

        $unseralizedObject = self::loadObject($object);
        $taskName = $unseralizedObject["task_general"]["task_name"];

        $select = $db->select()->from('tasks')
        ->where('id = ?', $id);

        $result = $db->fetchRow($select);


        $uuid = $result['guid'];
        $userId = $result['user_id'];
        $adminTask = $result['admin_task'];
        $unseralizedObject['uuid'] = $uuid;

        $converer = new Custom_ObjToTbt();


        $row = array(
        "guid" => $uuid,
        "user_id" => $userId,
        "data" => $object,
        "data_md5" => md5($object),
        "task_name" => $taskName,
        "admin_task" => $adminTask,
        "next_run_time" => $converer->CalcNextRunTime($unseralizedObject["task_scheduler"]),
        "task_type" => $unseralizedObject["task_destination"]["backupDestinationType"],
        "task_hidden" => 'No',
        "store_path" => $unseralizedObject["task_destination"]["localStorePath"]
        );
        $where = $db->quoteInto('id = ?', $id);

        $cs = new Custom_ComputerSelect();
        $taskChildren = $cs->getTaskComputers($id);
        //var_dump($taskChildren);
        //$insertRow = ();
        foreach($taskChildren as $computer)  {
            try {
                $insertRow = array("comp_guid" => $computer, "cmd_name" => "ADMIN_TASK_NEW", "status" => "pending", "xml" => "", "task_id" => "$id");
                self::setTaskStatus($uuid, "Stopped", 0, $computer, NULL, NULL, NULL, $converer->CalcNextRunTime($unseralizedObject["task_scheduler"]), 'no');
                $db->insert('cmdqueue', $insertRow);
            } catch (Exception $e) {
                Zend::dump($e);
            }
        }
        try {
            $return = $db->update('tasks', $row, $where);
        } catch (Exception $e) {
            Zend::dump($e);
        }
        return $return;
    }

    /**
     * Retrives a task from the database
     *
     * @param int $id
     * @return array
     */
    public static function getTaskObject($id) {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks')->where('id = ?', $id);
        $result = $db->fetchRow($select);

        $ob = self::loadObject($result["data"]);
        $ob["uuid"] = $result['guid'];
        $result['data'] = base64_encode(serialize($ob));
        return $result['data'];
    }

    
    /**
     * Retrives the XML tbt of a user task
     *
     * @param int $id
     * @return string
     */
    public static function getUserTaskObject($id) {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks')->where('id = ?', $id);
        $result = $db->fetchRow($select);
        return $result['data'];
    }
    
    /**
     * Retrives a lit of tasks
     *
     * @param string $order Order by
     * @param int $offset
     * @param string $search
     * @return array
     */
    public static function listTasks($order='id', $adminTask=false,$offset=0, $search=null, $orderType="ASC") {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks')
        ->order($order . " $orderType")
        ->limit(50,$offset);
        if ($adminTask)
        $select->where('admin_task = ?', "Yes");

        if ($search != null)
        $select->where('task_name LIKE ?', "%$search%");

        $select->where('task_hidden != ?', 'Yes');

        $result = $db->fetchAssoc($select);
        foreach ($result as $row) {
            $ap = $db->fetchRow("SELECT COUNT(rule_id) AS TOTAL FROM rules WHERE task_id = $row[id]");
            try {
                $que = $db->fetchRow("SELECT COUNT(task_id) AS TOTAL FROM cmdqueue WHERE task_id = $row[id] AND status != 'sended'");
            } catch (Exception $e) {
                Zend::dump($e);
            }
            $row["applyed"] = $ap["total"];
            $row["commands"] = $que["total"];
            $fResult[] = $row;
        }
        return $fResult;
    }

    /**
     * Deletes a task and its contents from the database
     *
     * @param int $id
     * @return int $rowsDeleted
     */
    public static function deleteTask($id, $guid=true) {
        $db = Zend::registry('dbController');
        $where = $db->quoteInto('id = ?', $id);
        $where2 = $db->quoteInto('task_id = ?', $id);
        /* Mark task as deleted */
        if ($guid != false) {
            $db->delete('cmdqueue', $db->quoteInto('task_id = ?', $id));
            $cs = new Custom_ComputerSelect();
            $comps = $cs->getTaskComputers($id);
            $taskGuid = $db->fetchCol('SELECT guid FROM tasks WHERE id = :id', array("id" => $id));
            $taskName = $db->fetchCol('SELECT task_name FROM tasks WHERE id = :id', array("id" => $id));
            try {
                $db->delete('task_status', $db->quoteInto('task_guid = ?', $taskGuid[0]));
            }catch (Exception $e) {
                var_dump($e);

            }
            foreach ($comps as $computer) {
                $xml = "<serverCommand><name>ADMIN-TASK-DELETE</name><tasks><task><taskName>$taskName[0]</taskName><taskID>$taskGuid[0]</taskID></task></tasks></serverCommand>";
                $insertRow = array("comp_guid" => $computer, "cmd_name" => "ADMIN_TASK_DELETE", "status" => "pending", "xml" => $xml, "task_id" => $id);
                $db->insert('cmdqueue', $insertRow);
            }
        } else {
            $tasid = $db->fetchCol('SELECT id FROM tasks WHERE guid = :guid', array("guid" => $id));
            $task_id = $tasid[0];
            $db->delete('cmdqueue', $db->quoteInto('task_id = ?', $task_id));
            $cs = new Custom_ComputerSelect();
            $comps = $cs->getTaskComputers($task_id);
            $taskGuid = $db->fetchCol('SELECT guid FROM tasks WHERE id = :id', array("id" => $task_id));
            $taskName = $db->fetchCol('SELECT task_name FROM tasks WHERE id = :id', array("id" => $task_id));
            $db->delete('task_status', $db->quoteInto('task_guid = ?', $taskGuid[0]));
            foreach ($comps as $computer) {
                $xml = "<serverCommand><name>ADMIN-TASK-DELETE</name><tasks><task><taskName>$taskName[0]</taskName><taskID>$taskGuid[0]</taskID></task></tasks></serverCommand>";
                $insertRow = array("comp_guid" => $computer, "cmd_name" => "ADMIN_TASK_DELETE", "status" => "pending", "xml" => $xml, "task_id" => $task_id);
                $db->insert('cmdqueue', $insertRow);
            }
        }

        if ($guid != true) {
            $fGuid = $db->fetchCol("SELECT guid FROM tasks WHERE id = :id", $id);
            $where3 = $db->quoteInto("task_guid = ?", $fGuid[0]);
            $db->delete('task_status', $where3);
        }
        $db->delete('rules', $where2);
        $histWhere = $db->quoteInto('task_id = ?', $guid==true?$id:$task_id);
        $db->delete('task_history', $histWhere);
        return $db->delete('tasks', $where);
    }

    public static function getTaskStatus($guid, $owner=NULL) {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('task_status')->where("task_guid = ?", $guid);
        $select->where('computer_guid = ?', $owner);
        $result = $db->fetchRow($select);
        if (!$result) {
            $select = $db->select()->from('tasks')->where("guid = ?", $guid);
            $result = $db->fetchRow($select);
        }
        return $result;
    }
    
    public static function setTaskStatus($guid, $status, $percentage, $owner, $last_run_time = NULL, $next_run_time = NULL, $last_completed_time = NULL, $nextRunTime = NULL, $missed=NULL) {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks')->where('guid = ?', $guid);
        $row = $db->fetchRow($select);
        $dateFormat = "%d/%m/%Y %H:%M:%S";
        $converter = new Custom_ObjToTbt();
        $taskObj = Custom_TaskManagement::loadObject($row["data"]);

/* The calc next run time has been commented to to implementation of NextRunTimeTag inside Reports  XML*/		
		
        $where = $db->quoteInto('guid = ?', $guid);
		if ($nextRunTime != NULL) { $row['next_run_time'] = $nextRunTime; }
        if ($last_run_time != NULL) $row["last_run_time"] = strftime($dateFormat, time());
        if ($last_completed_time != NULL) $row["last_completed_time"] = strftime($dateFormat, time());
        $row["update_time"] = strftime("%Y-%m-%d %H:%M:%S", time());
        $db->update('tasks', $row, $where);

        if ($row["admin_task"] == "No") {
            $where = $db->quoteInto('guid = ?', $guid);
            if ($status == "Stopping") $where .=  $db->quoteInto(" AND task_status != ?", 'Stopped');
            $row = array("guid" => "$guid", "owner" => "$owner", "task_status" => $status, "task_percentage" => $percentage);
		    if ($nextRunTime != NULL) { $row['next_run_time'] = $nextRunTime; }
            
            if ($last_run_time != NULL) $row["last_run_time"] = strftime($dateFormat, time());
            if ($last_completed_time != NULL) $row["last_completed_time"] = strftime($dateFormat, time());
	        $row["update_time"] = strftime("%Y-%m-%d %H:%M:%S", time());
	        $db->update('tasks', $row, $where);
        } else {
            $select2 = $db->select()->from('task_status')->where('task_guid = ?', $guid);
            $select2->where('computer_guid = ?', $owner);
            $row2 = $db->fetchAll($select2);
            if (count($row2) > 0) {
                $where1 = $db->quoteInto('task_guid = ?', $guid);
                $where2 = $db->quoteInto('computer_guid = ?', $owner);
                $where3 = "";
                if ($status == "Stopping") $where3 =  $db->quoteInto(" AND task_status != ?", 'Stopped');               
                $row = array("task_guid" => "$guid", "computer_guid" => "$owner", "task_status" => $status, "task_progress" => $percentage);
        		if ($nextRunTime != NULL) { $row['next_run_time'] = $nextRunTime; }               
                if ($last_run_time != NULL) $row["last_run_time"] = strftime($dateFormat, time());
                if ($last_completed_time != NULL) $row["last_completed_time"] = strftime($dateFormat, time());
                $row["update_time"] = strftime("%Y-%m-%d %H:%M:%S", time());
                if ($missed != NULL) $row["missed"] = $missed;                        
                $db->update('task_status', $row, "(($where1) AND ($where2))$where3");
            } else {
                $row = array("task_guid" => "$guid", "computer_guid" => "$owner", "task_status" => $status, "task_progress" => $percentage);
                if ($last_run_time != NULL) $row["last_run_time"] = strftime($dateFormat, time());
		        if ($nextRunTime != NULL) { $row['next_run_time'] = $nextRunTime; }                
                if ($last_completed_time != NULL) $row["last_completed_time"] = strftime($dateFormat, time());
                if ($missed != NULL) $row["missed"] = $missed;                
                if ($status != "Stopping") $db->insert('task_status', $row);
            }
        }
        return true;
    }


    public static function getPlugins() {
        $dir = 'plugins';
        $finalArray = array();
        $fdDir = opendir($dir);
        while (($file = readdir($fdDir))!==false) {
            if (eregi("\.xml$", $file) == false) continue;
            $xml = simplexml_load_file("./plugins/$file");
            $treeName = $xml->tree;
            $groupName = $xml->group;
            $plugName = $xml->name;
            $category = $xml->category;
            /*            if ($groupName == "") {
            $groupName = $plugName;
            $finalArray["$treeName"]["$groupName"] = array("plugin_name" => "$xml->name", "plugin_id" => "$xml->id", "plugin_file" => "$file", "root" => true);
            continue;
            }
            $finalArray["$treeName"]["$groupName"]["$xml->name"] = array("plugin_name" => "$xml->name", "plugin_id" => "$xml->id", "plugin_file" => "$file", "priority" => "$xml->priority");
            */
            if ($treeName == "User") {
                if ($groupName == "") {
                    $groupName = $plugName;
                    $finalArray["$treeName"]["$category"]["$groupName"] = array("plugin_name" => "$xml->name", "plugin_id" => (string)$xml->id, "plugin_file" => "$file", "root" => true);
                    continue;
                }
                $finalArray["$treeName"]["$category"]["$groupName"]["$xml->name"] = array("plugin_name" => "$xml->name", "plugin_id" => "$xml->id", "plugin_file" => "$file", "priority" => "$xml->priority");
            } else {
                if ($groupName == "") {
                    $groupName = $plugName;
                    $finalArray["$treeName"]["$groupName"] = array("plugin_name" => "$xml->name", "plugin_id" => "$xml->id", "plugin_file" => "$file", "root" => true);
                    continue;
                }
                $finalArray["$treeName"]["$groupName"]["$xml->name"] = array("plugin_name" => "$xml->name", "plugin_id" => "$xml->id", "plugin_file" => "$file", "priority" => "$xml->priority");
            }
        }
        //Zend::dump($finalArray);
        return $finalArray;
    }

    public static function getTaskGuid($id) {
        $db = Zend::registry('dbController');
        $result = $db->fetchRow('SELECT guid FROM tasks WHERE id = :id LIMIT 1', array("id" => "$id"));
        return $result['guid'];
    }

    public static function getTaskGeneralStatus($guid) {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('task_status')->where('task_guid = ?', $guid)->where('task_status != ?', 'Stopped');
        $result = $db->fetchAll($select);
        if (sizeof($result) > 0) return true;
        return false;
    }

    public static function makeNewTaskName() {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks', 'task_name')->where('task_name LIKE ?', 'New Task%');
        $result = $db->fetchAll($select);
        $cTasks = count($result);
        $newTaskName = "New Task " . $cTasks;

        if ($cTasks <= 0)
        return "New Task";

        if (self::taskExists($newTaskName) == true) {
            while (self::taskExists($newTaskName)) {
                $cTasks++;
                $newTaskName = "New Task " . $cTasks;
            }
        }
        return $newTaskName;
    }

    public static function taskExists($taskName) {
        $db = Zend::registry('dbController');
        $select = $db->select()->from('tasks', 'task_name')->where('task_name = ?', $taskName);
        if ($db->fetchRow($select)) {
            return true;
        } else {
            return false;
        }
    }

}
